@@ -114,7 +114,9 @@ public enum PGProperty {
   BINARY_TRANSFER(
       "binaryTransfer",
       "true",
-      "Use binary format for sending and receiving data if possible"),
+      "Enable binary transfer for supported built-in types if possible. "
+          + "Setting this to false disables any binary transfer unless it's individually activated "
+          + "for each type with `binaryTransferEnable`."),
 
   /**
    * Comma separated list of types to disable binary transfer. Either OID numbers or names.
@@ -123,7 +125,8 @@ public enum PGProperty {
   BINARY_TRANSFER_DISABLE(
       "binaryTransferDisable",
       "",
-      "Comma separated list of types to disable binary transfer. Either OID numbers or names. Overrides values in the driver default set and values set with binaryTransferEnable."),
+      "Comma separated list of types to disable binary transfer. Either OID numbers or names. "
+          + "Overrides values in the driver default set and values set with binaryTransferEnable."),
 
   /**
    * Comma separated list of types to enable binary transfer. Either OID numbers or names
@@ -131,7 +134,7 @@ public enum PGProperty {
   BINARY_TRANSFER_ENABLE(
       "binaryTransferEnable",
       "",
-      "Comma separated list of types to enable binary transfer. Either OID numbers or names"),
+      "Comma separated list of types to enable binary transfer. Either OID numbers or names."),
 
   /**
    * Cancel command is sent out of band over its own connection, so cancel message can itself get
@@ -144,6 +147,18 @@ public enum PGProperty {
       "10",
       "The timeout that is used for sending cancel command."),
 
+  /**
+   * Channel binding is a method for the server to authenticate itself to the
+   * client. It is only supported over SSL connections with PostgreSQL 11 or later
+   * servers using the SCRAM authentication method.
+   */
+  CHANNEL_BINDING(
+      "channelBinding",
+      "prefer",
+      "This option controls the client's use of channel binding.",
+      false,
+      new String[] {"disable", "prefer", "require"}),
+
   /**
    * Determine whether SAVEPOINTS used in AUTOSAVE will be released per query or not
    */
@@ -155,8 +170,8 @@ public enum PGProperty {
       new String[]{"true", "false"}),
 
   /**
-   * <p>The timeout value used for socket connect operations. If connecting to the server takes longer
-   * than this value, the connection is broken.</p>
+   * The timeout value used for socket connect operations. If connecting to the server takes longer
+   * than this value, the connection is broken.
    *
    * <p>The timeout is specified in seconds and a value of zero means that it is disabled.</p>
    */
@@ -229,7 +244,9 @@ public enum PGProperty {
    * reaches the same connection that is being initialized. All of the startup parameters will be wrapped
    * in a transaction
    * Note this is off by default as pgbouncer in statement mode
+   * @deprecated since we can send the startup parameters as a multistatment transaction
    */
+  @Deprecated
   GROUP_STARTUP_PARAMETERS(
       "groupStartupParameters",
       "false",
@@ -261,15 +278,28 @@ public enum PGProperty {
       new String[]{"auto", "sspi", "gssapi"}),
 
   /**
-   * <p>After requesting an upgrade to SSL from the server there are reports of the server not responding due to a failover
+   * After requesting an upgrade to SSL from the server there are reports of the server not responding due to a failover
    * without a timeout here, the client can wait forever. The pattern for requesting a GSS encrypted connection is the same so we provide the same
-   * timeout mechanism This timeout will be set before the request and reset after </p>
+   * timeout mechanism This timeout will be set before the request and reset after
    */
   GSS_RESPONSE_TIMEOUT(
       "gssResponseTimeout",
       "5000",
       "Time in milliseconds we wait for a response from the server after requesting a GSS upgrade"),
 
+  /**
+   * Flag to enable/disable the obtaining the default GSS credentials from a pre-existing ccache,
+   * rather than using JAAS.  This also allows GSS to work in environments where the default
+   * kerberos principal a user has is not user@DEFAULT_REALM, but some other user (this is valid,
+   * and often the case in more advanced Kerberos setups).  Finally, this also means that if
+   * the "native" GSS implementation is used (i.e. the local system GSS libraries), all means of
+   * fetching the default credential are supported.  Currently, JAAS is pure java on Linux, and
+   * does not support the use of KCM (and only supports file-based ccaches and keytabs).
+   */
+  GSS_USE_DEFAULT_CREDS(
+      "gssUseDefaultCreds",
+      "false",
+      "Use the default GSS credentials the process already has, rather than a JAAS login"),
 
   /**
    * Enable mode to filter out the names of database objects for which the current user has no privileges
@@ -318,8 +348,8 @@ public enum PGProperty {
       "If disabled hosts are connected in the given order. If enabled hosts are chosen randomly from the set of suitable candidates"),
 
   /**
-   * <p>If this is set then the client side will bind to this address. This is useful if you need
-   * to choose which interface to connect to.</p>
+   * If this is set then the client side will bind to this address. This is useful if you need
+   * to choose which interface to connect to.
    */
   LOCAL_SOCKET_ADDRESS(
       "localSocketAddress",
@@ -383,6 +413,14 @@ public enum PGProperty {
       null,
       "Specifies size of buffer during fetching result set. Can be specified as specified size or percent of heap memory."),
 
+  /**
+   * Maximum amount of bytes buffered before sending to the backend, default is 8192.
+   */
+  MAX_SEND_BUFFER_SIZE(
+      "maxSendBufferSize",
+      "8192",
+      "Maximum amount of bytes buffered before sending to the backend"),
+
   /**
    * Specify 'options' connection initialization parameter.
    * The value of this parameter may contain spaces and other special characters or their URL representation.
@@ -428,9 +466,9 @@ public enum PGProperty {
       "Port of the PostgreSQL server (may be specified directly in the JDBC URL)"),
 
   /**
-   * <p>Specifies which mode is used to execute queries to database: simple means ('Q' execute, no parse, no bind, text mode only),
+   * Specifies which mode is used to execute queries to database: simple means ('Q' execute, no parse, no bind, text mode only),
    * extended means always use bind/execute messages, extendedForPrepared means extended for prepared statements only,
-   * extendedCacheEverything means use extended protocol and try cache every statement (including Statement.execute(String sql)) in a query cache.</p>
+   * extendedCacheEverything means use extended protocol and try cache every statement (including Statement.execute(String sql)) in a query cache.
    *
    * <p>This mode is meant for debugging purposes and/or for cases when extended protocol cannot be used (e.g. logical replication protocol)</p>
    */
@@ -475,7 +513,7 @@ public enum PGProperty {
    */
   PROTOCOL_VERSION(
       "protocolVersion",
-      null,
+      "3",
       "Force use of a particular protocol version when connecting, currently only version 3 is supported.",
       false,
       new String[]{"3"}),
@@ -524,12 +562,13 @@ public enum PGProperty {
       "Socket read buffer size"),
 
   /**
-   * <p>Connection parameter passed in the startup message. This parameter accepts two values; "true"
+   * Connection parameter passed in the startup message. This parameter accepts two values; "true"
    * and "database". Passing "true" tells the backend to go into walsender mode, wherein a small set
    * of replication commands can be issued instead of SQL statements. Only the simple query protocol
    * can be used in walsender mode. Passing "database" as the value instructs walsender to connect
    * to the database specified in the dbname parameter, which will allow the connection to be used
-   * for logical replication from that database.</p>
+   * for logical replication from that database.
+   *
    * <p>Parameter should be use together with {@link PGProperty#ASSUME_MIN_SERVER_VERSION} with
    * parameter &gt;= 9.4 (backend &gt;= 9.4)</p>
    */
@@ -627,9 +666,7 @@ public enum PGProperty {
 
   /**
    * The String argument to give to the constructor of the SSL Factory.
-   * @deprecated use {@code ..Factory(Properties)} constructor.
    */
-  @Deprecated
   SSL_FACTORY_ARG(
       "sslfactoryarg",
       null,
@@ -665,6 +702,24 @@ public enum PGProperty {
       false,
       new String[]{"disable", "allow", "prefer", "require", "verify-ca", "verify-full"}),
 
+  /**
+   * Normally a GSS connection is attempted first. If this is set to {@code direct}
+   * then the GSS connection attempt will not be made
+   */
+  SSL_NEGOTIATION(
+      "sslNegotiation",
+      "postgres",
+      "This option controls whether the driver will perform its protocol\n"
+              + "negotiation to request encryption from the server or will just\n"
+              + "directly make a standard SSL connection. Traditional PostgreSQL\n"
+              + "protocol negotiation is the default and the most flexible with\n"
+              + "different server configurations. If the server is known to support\n"
+              + "direct SSL connections then the latter requires one\n"
+              + "fewer round trip reducing connection latency and also allows the use\n"
+              + "of protocol agnostic SSL network tools.",
+      false,
+      new String[]{"postgres", "direct"}),
+
   /**
    * The SSL password to use in the default CallbackHandler.
    */
@@ -683,8 +738,8 @@ public enum PGProperty {
       "A class, implementing javax.security.auth.callback.CallbackHandler that can handle PasswordCallback for the ssl password."),
 
   /**
-   * <p>After requesting an upgrade to SSL from the server there are reports of the server not responding due to a failover
-   * without a timeout here, the client can wait forever. This timeout will be set before the request and reset after </p>
+   * After requesting an upgrade to SSL from the server there are reports of the server not responding due to a failover
+   * without a timeout here, the client can wait forever. This timeout will be set before the request and reset after
    */
   SSL_RESPONSE_TIMEOUT(
       "sslResponseTimeout",
@@ -783,6 +838,7 @@ public enum PGProperty {
   private final /* @Nullable */ String defaultValue;
   private final boolean required;
   private final String description;
+  @SuppressWarnings("ImmutableEnumChecker")
   private final String /* @Nullable */ [] choices;
 
   PGProperty(String name, /* @Nullable */ String defaultValue, String description) {
