@@ -5,6 +5,8 @@
 
 package org.postgresql.core;
 
+import static org.postgresql.util.internal.Nullness.castNonNull;
+
 import org.postgresql.PGNotification;
 import org.postgresql.PGProperty;
 import org.postgresql.jdbc.AutoSave;
@@ -41,8 +43,9 @@ public abstract class QueryExecutorBase implements QueryExecutor {
   private final String database;
   private final int cancelSignalTimeout;
 
+  protected ProtocolVersion protocolVersion;
   private int cancelPid;
-  private int cancelKey;
+  private byte /* @Nullable */[] cancelKey;
   protected final QueryExecutorCloseAction closeAction;
   private /* @MonotonicNonNull */ String serverVersion;
   private int serverVersionNum;
@@ -75,6 +78,7 @@ public abstract class QueryExecutorBase implements QueryExecutor {
   @SuppressWarnings({"assignment", "argument", "method.invocation"})
   protected QueryExecutorBase(PGStream pgStream, int cancelSignalTimeout, Properties info) throws SQLException {
     this.pgStream = pgStream;
+    this.protocolVersion = pgStream.getProtocolVersion();
     this.user = PGProperty.USER.getOrDefault(info);
     this.database = PGProperty.PG_DBNAME.getOrDefault(info);
     this.cancelSignalTimeout = cancelSignalTimeout;
@@ -141,7 +145,7 @@ public abstract class QueryExecutorBase implements QueryExecutor {
     return database;
   }
 
-  public void setBackendKeyData(int cancelPid, int cancelKey) {
+  public void setBackendKeyData(int cancelPid, byte[]cancelKey) {
     this.cancelPid = cancelPid;
     this.cancelKey = cancelKey;
   }
@@ -190,16 +194,18 @@ public abstract class QueryExecutorBase implements QueryExecutor {
         LOGGER.log(Level.FINEST, " FE=> CancelRequest(pid={0},ckey={1})", new Object[]{cancelPid, cancelKey});
       }
 
+      // Cancel signal is variable since protocol 3.2 so we use cancelKey.length + 12
       cancelStream =
-          new PGStream(pgStream.getSocketFactory(), pgStream.getHostSpec(), cancelSignalTimeout);
+          new PGStream(pgStream.getSocketFactory(), pgStream.getHostSpec(), cancelSignalTimeout, castNonNull(cancelKey).length + 12);
       if (cancelSignalTimeout > 0) {
         cancelStream.setNetworkTimeout(cancelSignalTimeout);
       }
-      cancelStream.sendInteger4(16);
+      // send the length including self
+      cancelStream.sendInteger4(castNonNull(castNonNull(cancelKey)).length + 12);
       cancelStream.sendInteger2(1234);
       cancelStream.sendInteger2(5678);
       cancelStream.sendInteger4(cancelPid);
-      cancelStream.sendInteger4(cancelKey);
+      cancelStream.send(castNonNull(cancelKey));
       cancelStream.flush();
       cancelStream.receiveEOF();
     } catch (IOException e) {
@@ -264,7 +270,8 @@ public abstract class QueryExecutorBase implements QueryExecutor {
     if (serverVersionNum != 0) {
       return serverVersionNum;
     }
-    return serverVersionNum = Utils.parseServerVersionStr(getServerVersion());
+    serverVersionNum = Utils.parseServerVersionStr(getServerVersion());
+    return serverVersionNum;
   }
 
   public void setServerVersion(String serverVersion) {
@@ -395,6 +402,7 @@ public abstract class QueryExecutorBase implements QueryExecutor {
     return preferQueryMode;
   }
 
+  @Override
   public void setPreferQueryMode(PreferQueryMode mode) {
     preferQueryMode = mode;
   }
