@@ -75,6 +75,7 @@ public class MakeGSS {
       subjectDoAs = MethodHandles.lookup().findStatic(Subject.class, "doAs",
           MethodType.methodType(Object.class, Subject.class, PrivilegedAction.class));
     } catch (NoSuchMethodException | IllegalAccessException ignore) {
+      // E.g. Java 18+
     }
     SUBJECT_DO_AS = subjectDoAs;
 
@@ -83,6 +84,7 @@ public class MakeGSS {
       subjectCallAs = MethodHandles.lookup().findStatic(Subject.class, "callAs",
           MethodType.methodType(Object.class, Subject.class, Callable.class));
     } catch (NoSuchMethodException | IllegalAccessException ignore) {
+      // E.g. Java < 18
     }
     SUBJECT_CALL_AS = subjectCallAs;
   }
@@ -118,7 +120,7 @@ public class MakeGSS {
   public static void authenticate(boolean encrypted,
       PGStream pgStream, String host, String user, char /* @Nullable */ [] password,
       /* @Nullable */ String jaasApplicationName, /* @Nullable */ String kerberosServerName,
-      boolean useSpnego, boolean jaasLogin,
+      boolean useSpnego, boolean jaasLogin, boolean gssUseDefaultCreds,
       boolean logServerErrorDetail)
           throws IOException, PSQLException {
     LOGGER.log(Level.FINEST, " <=BE AuthenticationReqGSS");
@@ -151,10 +153,10 @@ public class MakeGSS {
       PrivilegedAction</* @Nullable */ Exception> action;
       if ( encrypted ) {
         action = new GssEncAction(pgStream, sub, host, user,
-            kerberosServerName, useSpnego, logServerErrorDetail);
+            kerberosServerName, useSpnego, gssUseDefaultCreds, logServerErrorDetail);
       } else {
         action = new GssAction(pgStream, sub, host, user,
-            kerberosServerName, useSpnego, logServerErrorDetail);
+            kerberosServerName, useSpnego, gssUseDefaultCreds, logServerErrorDetail);
       }
       //noinspection ConstantConditions
       @SuppressWarnings({"cast.unsafe", "assignment"})
