@@ -75,7 +75,9 @@ import java.time.OffsetDateTime;
 import java.time.OffsetTime;
 import java.util.ArrayList;
 import java.util.Calendar;
+import java.util.List;
 import java.util.Map;
+import java.util.StringJoiner;
 import java.util.TimeZone;
 import java.util.UUID;
 
@@ -124,7 +126,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         PSQLState.WRONG_OBJECT_TYPE);
   }
 
-  /*
+  /**
    * A Prepared SQL query is executed and its ResultSet is returned
    *
    * @return a ResultSet that contains the data produced by the * query - never null
@@ -504,24 +506,12 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
   @Override
   public void setBinaryStream(/* @Positive */ int parameterIndex, /* @Nullable */ InputStream x,
       /* @NonNegative */ int length) throws SQLException {
-    checkClosed();
-
-    if (x == null) {
-      setNull(parameterIndex, Types.VARBINARY);
-      return;
-    }
-
-    if (length < 0) {
-      throw new PSQLException(GT.tr("Invalid stream length {0}.", length),
-          PSQLState.INVALID_PARAMETER_VALUE);
-    }
-
     // Version 7.2 supports BinaryStream for the PG bytea type
     // As the spec/javadoc for this method indicate this is to be used for
     // large binary values (i.e. LONGVARBINARY) PG doesn't have a separate
     // long binary datatype, but with toast the bytea datatype is capable of
     // handling very large values.
-    preparedParameters.setBytea(parameterIndex, x, length);
+    setBinaryStream(parameterIndex, x, (long) length);
   }
 
   @Override
@@ -641,7 +631,10 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         } else {
           Date tmpd;
           if (in instanceof java.util.Date) {
-            tmpd = new Date(((java.util.Date) in).getTime());
+            // TODO: should we convert it to LocalDate instead?
+            @SuppressWarnings("JavaUtilDate")
+            Date res = new Date(((java.util.Date) in).getTime());
+            tmpd = res;
           } else if (in instanceof LocalDate) {
             setDate(parameterIndex, (LocalDate) in);
             break;
@@ -657,7 +650,10 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         } else {
           Time tmpt;
           if (in instanceof java.util.Date) {
-            tmpt = new Time(((java.util.Date) in).getTime());
+            // TODO: should we convert it to OffsetTime instead?
+            @SuppressWarnings("JavaUtilDate")
+            Time res = new Time(((java.util.Date) in).getTime());
+            tmpt = res;
           } else if (in instanceof LocalTime) {
             setTime(parameterIndex, (LocalTime) in);
             break;
@@ -678,12 +674,16 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         } else {
           Timestamp tmpts;
           if (in instanceof java.util.Date) {
-            tmpts = new Timestamp(((java.util.Date) in).getTime());
+            // TODO: should we convert it to LocalDateTime instead?
+            @SuppressWarnings("JavaUtilDate")
+            Timestamp res = new Timestamp(((java.util.Date) in).getTime());
+            tmpts = res;
           } else if (in instanceof LocalDateTime) {
             setTimestamp(parameterIndex, (LocalDateTime) in);
             break;
           } else {
-            tmpts = getTimestampUtils().toTimestamp(getDefaultCalendar(), in.toString());
+            Charset connectionCharset = Charset.forName(connection.getEncoding().name());
+            tmpts = getTimestampUtils().toTimestamp(getDefaultCalendar(), in.toString().getBytes(connectionCharset));
           }
           setTimestamp(parameterIndex, tmpts);
         }
@@ -763,7 +763,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     }
   }
 
-  private Class<?> getArrayType(Class<?> type) {
+  private static Class<?> getArrayType(Class<?> type) {
     Class<?> subType = type.getComponentType();
     while (subType != null) {
       type = subType;
@@ -810,7 +810,9 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         return ((Number) in).intValue();
       }
       if (in instanceof java.util.Date) {
-        return (int) ((java.util.Date) in).getTime();
+        @SuppressWarnings("JavaUtilDate")
+        long time = ((java.util.Date) in).getTime();
+        return (int) time;
       }
       if (in instanceof Boolean) {
         return (Boolean) in ? 1 : 0;
@@ -836,7 +838,9 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         return ((Number) in).shortValue();
       }
       if (in instanceof java.util.Date) {
-        return (short) ((java.util.Date) in).getTime();
+        @SuppressWarnings("JavaUtilDate")
+        long time = ((java.util.Date) in).getTime();
+        return (short) time;
       }
       if (in instanceof Boolean) {
         return (Boolean) in ? (short) 1 : (short) 0;
@@ -862,7 +866,9 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         return ((Number) in).longValue();
       }
       if (in instanceof java.util.Date) {
-        return ((java.util.Date) in).getTime();
+        @SuppressWarnings("JavaUtilDate")
+        long time = ((java.util.Date) in).getTime();
+        return time;
       }
       if (in instanceof Boolean) {
         return (Boolean) in ? 1L : 0L;
@@ -888,7 +894,9 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         return ((Number) in).floatValue();
       }
       if (in instanceof java.util.Date) {
-        return ((java.util.Date) in).getTime();
+        @SuppressWarnings("JavaUtilDate")
+        long time = ((java.util.Date) in).getTime();
+        return time;
       }
       if (in instanceof Boolean) {
         return (Boolean) in ? 1f : 0f;
@@ -914,7 +922,9 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         return ((Number) in).doubleValue();
       }
       if (in instanceof java.util.Date) {
-        return ((java.util.Date) in).getTime();
+        @SuppressWarnings("JavaUtilDate")
+        long time = ((java.util.Date) in).getTime();
+        return time;
       }
       if (in instanceof Boolean) {
         return (Boolean) in ? 1d : 0d;
@@ -946,7 +956,9 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       } else if (in instanceof Double || in instanceof Float) {
         rc = BigDecimal.valueOf(((Number) in).doubleValue());
       } else if (in instanceof java.util.Date) {
-        rc = BigDecimal.valueOf(((java.util.Date) in).getTime());
+        @SuppressWarnings("JavaUtilDate")
+        long time = ((java.util.Date) in).getTime();
+        rc = BigDecimal.valueOf(time);
       } else if (in instanceof Boolean) {
         rc = (Boolean) in ? BigDecimal.ONE : BigDecimal.ZERO;
       } else if (in instanceof Clob) {
@@ -1090,7 +1102,25 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       return super.toString();
     }
 
-    return preparedQuery.query.toString(preparedParameters);
+    List<Query> batchStatements = this.batchStatements;
+    if (batchStatements == null || batchStatements.isEmpty()) {
+      return preparedQuery.query.toString(preparedParameters);
+    }
+    List</* @Nullable */ ParameterList> batchParameters = this.batchParameters;
+    StringJoiner sj = new StringJoiner(";\n");
+    if (batchStatements.size() == 1 && batchParameters != null) {
+      // For rewritebatchinserts=true case, we have a single batch statement and multiple parameter rows
+      Query query = batchStatements.get(0);
+      for (ParameterList batchParameter : batchParameters) {
+        sj.add(query.toString(batchParameter));
+      }
+    } else {
+      for (int i = 0; i < batchStatements.size(); i++) {
+        Query statement = batchStatements.get(i);
+        sj.add(statement.toString(batchParameters == null ? null : batchParameters.get(i)));
+      }
+    }
+    return sj.toString();
   }
 
   /**
@@ -1256,19 +1286,23 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       return;
     }
 
-    InputStream inStream = x.getBinaryStream();
-    try {
-      long oid = createBlob(i, inStream, x.length());
-      setLong(i, oid);
-    } finally {
-      try {
-        inStream.close();
-      } catch (Exception e) {
+    try (InputStream inStream = x.getBinaryStream(); ) {
+      long maxLength = x.length();
+      if (maxLength < 0) {
+        // Hibernate used to create blob instances that report -1 length, so we ignore the length
+        // and assume we need to read all the data from the stream.
+        // See https://github.com/pgjdbc/pgjdbc/issues/3134
+        maxLength = Long.MAX_VALUE;
       }
+      long oid = createBlob(i, inStream, maxLength);
+      setLong(i, oid);
+    } catch (IOException e) {
+      throw new PSQLException(GT.tr("Unexpected error when closing Blob binary stream"),
+          PSQLState.UNEXPECTED_ERROR, e);
     }
   }
 
-  private String readerToString(Reader value, int maxLength) throws SQLException {
+  private static String readerToString(Reader value, int maxLength) throws SQLException {
     try {
       int bufferSize = Math.min(maxLength, 1024);
       StringBuilder v = new StringBuilder(bufferSize);
@@ -1321,6 +1355,10 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
 
     Reader inStream = x.getCharacterStream();
     int length = (int) x.length();
+    if (length < 0) {
+      // See #setBlob(int, Blob)
+      length = Integer.MAX_VALUE;
+    }
     LargeObjectManager lom = connection.getLargeObjectAPI();
     long oid = lom.createLO();
     LargeObject lob = lom.open(oid);
@@ -1589,16 +1627,23 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
   public void setBinaryStream(/* @Positive */ int parameterIndex, /* @Nullable */ InputStream value,
       /* @NonNegative */ /* @IntRange(from = 0, to = Integer.MAX_VALUE) */ long length)
       throws SQLException {
+    checkClosed();
+
+    if (value == null) {
+      setNull(parameterIndex, Types.VARBINARY);
+      return;
+    }
+
     //noinspection ConstantConditions
     if (length > Integer.MAX_VALUE) {
       throw new PSQLException(GT.tr("Object is too large to send over the protocol."),
           PSQLState.NUMERIC_CONSTANT_OUT_OF_RANGE);
+    } else if (length < 0) {
+      throw new PSQLException(GT.tr("Invalid stream length {0}.", length),
+          PSQLState.INVALID_PARAMETER_VALUE);
     }
-    if (value == null) {
-      preparedParameters.setNull(parameterIndex, Oid.BYTEA);
-    } else {
-      preparedParameters.setBytea(parameterIndex, value, (int) length);
-    }
+
+    preparedParameters.setBytea(parameterIndex, value, (int) length);
   }
 
   @Override
