@@ -71,6 +71,7 @@ public class PgStatement implements Statement, BaseStatement {
    * cancelTask was created. Note: the field must be set/get/compareAndSet via
    * {@link #CANCEL_TIMER_UPDATER} as per {@link AtomicReferenceFieldUpdater} javadoc.
    */
+  @SuppressWarnings("unused")
   private volatile /* @Nullable */ TimerTask cancelTimerTask;
 
   @SuppressWarnings("RedundantCast")
@@ -161,11 +162,26 @@ public class PgStatement implements Statement, BaseStatement {
       throws SQLException {
     this.connection = c;
     forceBinaryTransfers |= c.getForceBinary();
+    // validation check for allowed values of resultset type
+    if (rsType != ResultSet.TYPE_FORWARD_ONLY && rsType != ResultSet.TYPE_SCROLL_INSENSITIVE && rsType != ResultSet.TYPE_SCROLL_SENSITIVE) {
+      throw new PSQLException(GT.tr("Unknown value for ResultSet type"),
+          PSQLState.INVALID_PARAMETER_VALUE);
+    }
     resultsettype = rsType;
+    // validation check for allowed values of resultset concurrency
+    if (rsConcurrency != ResultSet.CONCUR_READ_ONLY && rsConcurrency != ResultSet.CONCUR_UPDATABLE) {
+      throw new PSQLException(GT.tr("Unknown value for ResultSet concurrency"),
+          PSQLState.INVALID_PARAMETER_VALUE);
+    }
     concurrency = rsConcurrency;
     setFetchSize(c.getDefaultFetchSize());
     setPrepareThreshold(c.getPrepareThreshold());
     setAdaptiveFetch(c.getAdaptiveFetch());
+    // validation check for allowed values of resultset holdability
+    if (rsHoldability != ResultSet.HOLD_CURSORS_OVER_COMMIT && rsHoldability != ResultSet.CLOSE_CURSORS_AT_COMMIT) {
+      throw new PSQLException(GT.tr("Unknown value for ResultSet holdability"),
+          PSQLState.INVALID_PARAMETER_VALUE);
+    }
     this.rsHoldability = rsHoldability;
   }
 
@@ -617,7 +633,7 @@ public class PgStatement implements Statement, BaseStatement {
   }
 
   /**
-   * <p>Either initializes new warning wrapper, or adds warning onto the chain.</p>
+   * Either initializes new warning wrapper, or adds warning onto the chain.
    *
    * <p>Although warnings are expected to be added sequentially, the warnings chain may be cleared
    * concurrently at any time via {@link #clearWarnings()}, therefore it is possible that a warning
@@ -660,7 +676,8 @@ public class PgStatement implements Statement, BaseStatement {
   }
 
   /**
-   * <p>Clears the warning chain.</p>
+   * Clears the warning chain.
+   *
    * <p>Note that while it is safe to clear warnings while the query is executing, warnings that are
    * added between calls to {@link #getWarnings()} and #clearWarnings() may be missed.
    * Therefore you should hold a reference to the tail of the previous warning chain
@@ -816,10 +833,6 @@ public class PgStatement implements Statement, BaseStatement {
 
     int flags;
 
-    // Force a Describe before any execution? We need to do this if we're going
-    // to send anything dependent on the Describe results, e.g. binary parameters.
-    boolean preDescribe = false;
-
     if (wantsGeneratedKeysAlways) {
       /*
        * This batch will return generated keys, tell the executor to expect result rows. We also
@@ -844,27 +857,9 @@ public class PgStatement implements Statement, BaseStatement {
       flags |= QueryExecutor.QUERY_EXECUTE_AS_SIMPLE;
     }
 
-    boolean sameQueryAhead = queries.length > 1 && queries[0] == queries[1];
-
-    if (!sameQueryAhead
-        // If executing the same query twice in a batch, make sure the statement
-        // is server-prepared. In other words, "oneshot" only if the query is one in the batch
-        // or the queries are different
-        || isOneShotQuery(null)) {
+    if (isOneShotQuery(null)) {
       flags |= QueryExecutor.QUERY_ONESHOT;
     } else {
-      // If a batch requests generated keys and isn't already described,
-      // force a Describe of the query before proceeding. That way we can
-      // determine the appropriate size of each batch by estimating the
-      // maximum data returned. Without that, we don't know how many queries
-      // we'll be able to queue up before we risk a deadlock.
-      // (see v3.QueryExecutorImpl's MAX_BUFFERED_RECV_BYTES)
-
-      // SameQueryAhead is just a quick way to issue pre-describe for batch execution
-      // TODO: It should be reworked into "pre-describe if query has unknown parameter
-      // types and same query is ahead".
-      preDescribe = (wantsGeneratedKeysAlways || sameQueryAhead)
-          && !queries[0].isStatementDescribed();
       /*
        * It's also necessary to force a Describe on the first execution of the new statement, even
        * though we already described it, to work around bug #267.
@@ -882,27 +877,6 @@ public class PgStatement implements Statement, BaseStatement {
     BatchResultHandler handler;
     handler = createBatchHandler(queries, parameterLists);
 
-    if ((preDescribe || forceBinaryTransfers)
-        && (flags & QueryExecutor.QUERY_EXECUTE_AS_SIMPLE) == 0) {
-      // Do a client-server round trip, parsing and describing the query so we
-      // can determine its result types for use in binary parameters, batch sizing,
-      // etc.
-      int flags2 = flags | QueryExecutor.QUERY_DESCRIBE_ONLY;
-      StatementResultHandler handler2 = new StatementResultHandler();
-      try {
-        connection.getQueryExecutor().execute(queries[0], parameterLists[0], handler2, 0, 0, flags2);
-      } catch (SQLException e) {
-        // Unable to parse the first statement -> throw BatchUpdateException
-        handler.handleError(e);
-        handler.handleCompletion();
-        // Will not reach here (see above)
-      }
-      ResultWrapper result2 = handler2.getResults();
-      if (result2 != null) {
-        castNonNull(result2.getResultSet(), "result2.getResultSet()").close();
-      }
-    }
-
     try (ResourceLock ignore = lock.obtain()) {
       result = null;
     }
@@ -995,7 +969,7 @@ public class PgStatement implements Statement, BaseStatement {
   public void setFetchSize(/* @NonNegative */ int rows) throws SQLException {
     checkClosed();
     if (rows < 0) {
-      throw new PSQLException(GT.tr("Fetch size must be a value greater to or equal to 0."),
+      throw new PSQLException(GT.tr("Fetch size must be a value greater than or equal to 0."),
           PSQLState.INVALID_PARAMETER_VALUE);
     }
     fetchSize = rows;
@@ -1027,7 +1001,8 @@ public class PgStatement implements Statement, BaseStatement {
         return;
       }
       cancel();
-    } catch (SQLException e) {
+    } catch (SQLException ignored) {
+      // We can't do much if the cancel fails
     }
   }
 
@@ -1137,8 +1112,11 @@ public class PgStatement implements Statement, BaseStatement {
     if (autoGeneratedKeys == Statement.NO_GENERATED_KEYS) {
       return executeLargeUpdate(sql);
     }
-
-    return executeLargeUpdate(sql, (String[]) null);
+    if (autoGeneratedKeys == Statement.RETURN_GENERATED_KEYS) {
+      return executeLargeUpdate(sql, (String[]) null);
+    }
+    throw new PSQLException(GT.tr("Invalid value for autoGeneratedKeys {0}", autoGeneratedKeys),
+        PSQLState.INVALID_PARAMETER_VALUE);
   }
 
   @Override
@@ -1275,8 +1253,11 @@ public class PgStatement implements Statement, BaseStatement {
     if (autoGeneratedKeys == Statement.NO_GENERATED_KEYS) {
       return executeUpdate(sql);
     }
-
-    return executeUpdate(sql, (String[]) null);
+    if (autoGeneratedKeys == Statement.RETURN_GENERATED_KEYS) {
+      return executeUpdate(sql, (String[]) null);
+    }
+    throw new PSQLException(GT.tr("Invalid value for autoGeneratedKeys {0}", autoGeneratedKeys),
+        PSQLState.INVALID_PARAMETER_VALUE);
   }
 
   @Override
@@ -1309,7 +1290,11 @@ public class PgStatement implements Statement, BaseStatement {
     if (autoGeneratedKeys == Statement.NO_GENERATED_KEYS) {
       return execute(sql);
     }
-    return execute(sql, (String[]) null);
+    if (autoGeneratedKeys == Statement.RETURN_GENERATED_KEYS) {
+      return execute(sql, (String[]) null);
+    }
+    throw new PSQLException(GT.tr("Invalid value for autoGeneratedKeys {0}", autoGeneratedKeys),
+        PSQLState.INVALID_PARAMETER_VALUE);
   }
 
   @Override
