@@ -26,10 +26,10 @@ import javax.net.ssl.TrustManagerFactory;
 import javax.net.ssl.X509TrustManager;
 
 /**
- * <p>Provides a SSLSocketFactory that authenticates the remote server against an explicit pre-shared
+ * Provides a SSLSocketFactory that authenticates the remote server against an explicit pre-shared
  * SSL certificate. This is more secure than using the NonValidatingFactory as it prevents "man in
  * the middle" attacks. It is also more secure than relying on a central CA signing your server's
- * certificate as it pins the server's certificate.</p>
+ * certificate as it pins the server's certificate.
  *
  * <p>This class requires a single String parameter specified by setting the connection property
  * <code>sslfactoryarg</code>. The value of this property is the PEM-encoded remote server's SSL
@@ -88,7 +88,7 @@ public class SingleCertValidatingFactory extends WrappedFactory {
   private static final String SYS_PROP_PREFIX = "sys:";
 
   public SingleCertValidatingFactory(String sslFactoryArg) throws GeneralSecurityException {
-    if (sslFactoryArg == null || sslFactoryArg.equals("")) {
+    if (sslFactoryArg == null || "".equals(sslFactoryArg)) {
       throw new GeneralSecurityException(GT.tr("The sslfactoryarg property may not be empty."));
     }
     InputStream in = null;
@@ -187,15 +187,18 @@ public class SingleCertValidatingFactory extends WrappedFactory {
       }
     }
 
+    @Override
     public void checkClientTrusted(X509Certificate[] chain, String authType)
         throws CertificateException {
     }
 
+    @Override
     public void checkServerTrusted(X509Certificate[] chain, String authType)
         throws CertificateException {
       trustManager.checkServerTrusted(chain, authType);
     }
 
+    @Override
     public X509Certificate[] getAcceptedIssuers() {
       return new X509Certificate[]{cert};
     }
