@@ -42,7 +42,7 @@ public class PGInterval extends PGobject implements Serializable, Cloneable {
   /**
    * Initialize a interval with a given interval string representation.
    *
-   * @param value String representated interval (e.g. '3 years 2 mons')
+   * @param value String represented interval (e.g. '3 years 2 mons')
    * @throws SQLException Is thrown if the string representation has an unknown format
    * @see PGobject#setValue(String)
    */
@@ -52,7 +52,7 @@ public class PGInterval extends PGobject implements Serializable, Cloneable {
     setValue(value);
   }
 
-  private int lookAhead(String value, int position, String find) {
+  private static int lookAhead(String value, int position, String find) {
     char [] tokens = find.toCharArray();
     int found = -1;
 
@@ -132,7 +132,7 @@ public class PGInterval extends PGobject implements Serializable, Cloneable {
    * Sets a interval string represented value to this instance. This method only recognize the
    * format, that Postgres returns - not all input formats are supported (e.g. '1 yr 2 m 3 s').
    *
-   * @param value String representated interval (e.g. '3 years 2 mons')
+   * @param value String represented interval (e.g. '3 years 2 mons')
    * @throws SQLException Is thrown if the string representation has an unknown format
    */
   @Override
@@ -165,6 +165,7 @@ public class PGInterval extends PGobject implements Serializable, Cloneable {
       String valueToken = null;
 
       value = value.replace('+', ' ').replace('@', ' ');
+      value = value.toLowerCase(Locale.ROOT);
       final StringTokenizer st = new StringTokenizer(value);
       for (int i = 1; st.hasMoreTokens(); i++) {
         String token = st.nextToken();
@@ -422,6 +423,7 @@ public class PGInterval extends PGobject implements Serializable, Cloneable {
    *
    * @param date Date instance to add to
    */
+  @SuppressWarnings("JavaUtilDate")
   public void add(Date date) {
     if (isNull) {
       return;
