@@ -5,7 +5,7 @@
 
 package org.postgresql.jdbc;
 
-import static org.junit.Assert.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 
 import org.postgresql.PGNotification;
 import org.postgresql.copy.CopyManager;
@@ -25,7 +25,7 @@ import org.postgresql.util.LruCache;
 import org.postgresql.util.PGobject;
 import org.postgresql.xml.PGXmlFactoryFactory;
 
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
 
 import java.lang.reflect.Array;
 import java.sql.Blob;
@@ -74,14 +74,14 @@ public abstract class AbstractArraysTest<A> {
 
   protected void assertArraysEquals(String message, A expected, Object actual) {
     final int expectedLength = Array.getLength(expected);
-    assertEquals(message + " size", expectedLength, Array.getLength(actual));
-    for (int i = 0; i < expectedLength; ++i) {
-      assertEquals(message + " value at " + i, Array.get(expected, i), Array.get(actual, i));
+    assertEquals(expectedLength, Array.getLength(actual), message + " size");
+    for (int i = 0; i < expectedLength; i++) {
+      assertEquals(Array.get(expected, i), Array.get(actual, i), message + " value at " + i);
     }
   }
 
   @Test
-  public void testBinary() throws Exception {
+  public void binary() throws Exception {
 
     A data = testData[0][0];
 
@@ -103,7 +103,7 @@ public abstract class AbstractArraysTest<A> {
   }
 
   @Test
-  public void testString() throws Exception {
+  public void string() throws Exception {
 
     A data = testData[0][0];
 
@@ -138,7 +138,7 @@ public abstract class AbstractArraysTest<A> {
 
       assertEquals(data.length, actual.length);
 
-      for (int i = 0; i < data.length; ++i) {
+      for (int i = 0; i < data.length; i++) {
         assertArraysEquals("array at position " + i, data[i], actual[i]);
       }
     }
@@ -159,7 +159,7 @@ public abstract class AbstractArraysTest<A> {
 
     assertEquals(data.length, actual.length);
 
-    for (int i = 0; i < data.length; ++i) {
+    for (int i = 0; i < data.length; i++) {
       assertArraysEquals("array at position " + i, data[i], actual[i]);
     }
   }
@@ -182,9 +182,9 @@ public abstract class AbstractArraysTest<A> {
 
       assertEquals(testData.length, actual.length);
 
-      for (int i = 0; i < testData.length; ++i) {
-        assertEquals("array length at " + i, testData[i].length, actual[i].length);
-        for (int j = 0; j < testData[i].length; ++j) {
+      for (int i = 0; i < testData.length; i++) {
+        assertEquals(testData[i].length, actual[i].length, "array length at " + i);
+        for (int j = 0; j < testData[i].length; j++) {
           assertArraysEquals("array at " + i + ',' + j, testData[i][j], actual[i][j]);
         }
       }
@@ -204,18 +204,18 @@ public abstract class AbstractArraysTest<A> {
 
     assertEquals(testData.length, actual.length);
 
-    for (int i = 0; i < testData.length; ++i) {
-      assertEquals("array length at " + i, testData[i].length, actual[i].length);
-      for (int j = 0; j < testData[i].length; ++j) {
+    for (int i = 0; i < testData.length; i++) {
+      assertEquals(testData[i].length, actual[i].length, "array length at " + i);
+      for (int j = 0; j < testData[i].length; j++) {
         assertArraysEquals("array at " + i + ',' + j, testData[i][j], actual[i][j]);
       }
     }
   }
 
   @Test
-  public void testObjectArrayCopy() throws Exception {
+  public void objectArrayCopy() throws Exception {
     final Object[] copy = new Object[testData.length];
-    for (int i = 0; i < testData.length; ++i) {
+    for (int i = 0; i < testData.length; i++) {
       copy[i] = testData[i];
     }
 
@@ -229,9 +229,9 @@ public abstract class AbstractArraysTest<A> {
   }
 
   @Test
-  public void testObject2dArrayCopy() throws Exception {
+  public void object2dArrayCopy() throws Exception {
     final Object[][] copy = new Object[testData.length][];
-    for (int  i = 0; i < testData.length; ++i) {
+    for (int  i = 0; i < testData.length; i++) {
       copy[i] = testData[i];
     }
 
@@ -245,11 +245,11 @@ public abstract class AbstractArraysTest<A> {
   }
 
   @Test
-  public void testObject3dArrayCopy() throws Exception {
+  public void object3dArrayCopy() throws Exception {
     final A[][][] source = (A[][][]) Array.newInstance(testData.getClass(), 2);
     source[0] = testData;
     source[1] = testData;
-    final Object[][][] copy = new Object[][][] { testData, testData };
+    final Object[][][] copy = new Object[][][]{testData, testData};
 
     final ArrayEncoding.ArrayEncoder<A[][][]> support = ArrayEncoding.getArrayEncoder(source);
     final String arrayString = support.toArrayString(',', source);
@@ -271,6 +271,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Encoding getEncoding() throws SQLException {
       return encoding;
     }
@@ -278,6 +279,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public TypeInfo getTypeInfo() {
       return typeInfo;
     }
@@ -285,6 +287,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void cancelQuery() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -292,6 +295,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public ResultSet execSQLQuery(String s) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -299,6 +303,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public ResultSet execSQLQuery(String s, int resultSetType, int resultSetConcurrency) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -306,6 +311,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void execSQLUpdate(String s) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -313,6 +319,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public QueryExecutor getQueryExecutor() {
       throw new UnsupportedOperationException();
     }
@@ -320,6 +327,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public ReplicationProtocol getReplicationProtocol() {
       throw new UnsupportedOperationException();
     }
@@ -327,6 +335,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Object getObject(String type, String value, byte[] byteValue) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -334,6 +343,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean haveMinimumServerVersion(int ver) {
       throw new UnsupportedOperationException();
     }
@@ -341,6 +351,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean haveMinimumServerVersion(Version ver) {
       throw new UnsupportedOperationException();
     }
@@ -348,6 +359,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public byte[] encodeString(String str) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -355,6 +367,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String escapeString(String str) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -362,6 +375,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getStandardConformingStrings() {
       throw new UnsupportedOperationException();
     }
@@ -369,6 +383,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public TimestampUtils getTimestampUtils() {
       throw new UnsupportedOperationException();
     }
@@ -376,6 +391,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Logger getLogger() {
       throw new UnsupportedOperationException();
     }
@@ -383,6 +399,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getStringVarcharFlag() {
       throw new UnsupportedOperationException();
     }
@@ -390,6 +407,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public TransactionState getTransactionState() {
       throw new UnsupportedOperationException();
     }
@@ -397,6 +415,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean binaryTransferSend(int oid) {
       throw new UnsupportedOperationException();
     }
@@ -404,6 +423,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isColumnSanitiserDisabled() {
       throw new UnsupportedOperationException();
     }
@@ -411,6 +431,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void addTimerTask(TimerTask timerTask, long milliSeconds) {
       throw new UnsupportedOperationException();
     }
@@ -418,6 +439,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void purgeTimerTasks() {
       throw new UnsupportedOperationException();
     }
@@ -425,6 +447,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public LruCache<Key, FieldMetadata> getFieldMetadataCache() {
       throw new UnsupportedOperationException();
     }
@@ -432,6 +455,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public CachedQuery createQuery(String sql, boolean escapeProcessing, boolean isParameterized, String... columnNames)
         throws SQLException {
       throw new UnsupportedOperationException();
@@ -440,6 +464,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setFlushCacheOnDeallocate(boolean flushCacheOnDeallocate) {
       throw new UnsupportedOperationException();
     }
@@ -447,6 +472,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Statement createStatement() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -454,6 +480,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PreparedStatement prepareStatement(String sql) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -461,6 +488,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public CallableStatement prepareCall(String sql) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -468,6 +496,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String nativeSQL(String sql) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -475,6 +504,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setAutoCommit(boolean autoCommit) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -482,6 +512,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getAutoCommit() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -489,6 +520,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void commit() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -496,6 +528,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void rollback() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -503,6 +536,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void close() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -510,6 +544,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isClosed() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -517,6 +552,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public DatabaseMetaData getMetaData() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -524,6 +560,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setReadOnly(boolean readOnly) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -531,6 +568,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isReadOnly() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -538,6 +576,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setCatalog(String catalog) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -545,6 +584,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getCatalog() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -552,6 +592,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setTransactionIsolation(int level) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -559,6 +600,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public int getTransactionIsolation() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -566,6 +608,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public SQLWarning getWarnings() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -573,6 +616,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void clearWarnings() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -580,6 +624,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -587,6 +632,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
         throws SQLException {
       throw new UnsupportedOperationException();
@@ -595,6 +641,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -602,6 +649,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Map<String, Class<?>> getTypeMap() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -609,6 +657,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setTypeMap(Map<String, Class<?>> map) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -616,6 +665,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setHoldability(int holdability) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -623,6 +673,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public int getHoldability() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -630,6 +681,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Savepoint setSavepoint() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -637,6 +689,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Savepoint setSavepoint(String name) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -644,6 +697,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void rollback(Savepoint savepoint) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -651,6 +705,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void releaseSavepoint(Savepoint savepoint) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -658,6 +713,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability)
         throws SQLException {
       throw new UnsupportedOperationException();
@@ -666,6 +722,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency,
         int resultSetHoldability) throws SQLException {
       throw new UnsupportedOperationException();
@@ -674,6 +731,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency,
         int resultSetHoldability) throws SQLException {
       throw new UnsupportedOperationException();
@@ -682,6 +740,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -689,6 +748,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -696,6 +756,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -703,6 +764,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Clob createClob() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -710,6 +772,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Blob createBlob() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -717,6 +780,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public NClob createNClob() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -724,6 +788,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public SQLXML createSQLXML() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -731,6 +796,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isValid(int timeout) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -738,6 +804,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setClientInfo(String name, String value) throws SQLClientInfoException {
       throw new UnsupportedOperationException();
     }
@@ -745,6 +812,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setClientInfo(Properties properties) throws SQLClientInfoException {
       throw new UnsupportedOperationException();
     }
@@ -752,6 +820,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getClientInfo(String name) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -759,6 +828,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Properties getClientInfo() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -766,6 +836,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public java.sql.Array createArrayOf(String typeName, Object[] elements) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -773,6 +844,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -780,6 +852,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setSchema(String schema) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -787,6 +860,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getSchema() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -794,6 +868,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void abort(Executor executor) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -801,6 +876,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setNetworkTimeout(Executor executor, int milliseconds) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -808,6 +884,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public int getNetworkTimeout() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -815,6 +892,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public <T> T unwrap(Class<T> iface) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -822,6 +900,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isWrapperFor(Class<?> iface) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -829,6 +908,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public java.sql.Array createArrayOf(String typeName, Object elements) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -836,6 +916,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PGNotification[] getNotifications() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -843,6 +924,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PGNotification[] getNotifications(int timeoutMillis) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -850,6 +932,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public CopyManager getCopyAPI() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -857,6 +940,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public LargeObjectManager getLargeObjectAPI() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -864,6 +948,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public Fastpath getFastpathAPI() throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -871,6 +956,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void addDataType(String type, String className) {
       throw new UnsupportedOperationException();
     }
@@ -878,6 +964,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void addDataType(String type, Class<? extends PGobject> klass) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -885,6 +972,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setPrepareThreshold(int threshold) {
       throw new UnsupportedOperationException();
     }
@@ -892,6 +980,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public int getPrepareThreshold() {
       throw new UnsupportedOperationException();
     }
@@ -899,6 +988,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setDefaultFetchSize(int fetchSize) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -906,6 +996,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public int getDefaultFetchSize() {
       throw new UnsupportedOperationException();
     }
@@ -913,6 +1004,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public int getBackendPID() {
       throw new UnsupportedOperationException();
     }
@@ -920,6 +1012,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String escapeIdentifier(String identifier) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -927,6 +1020,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String escapeLiteral(String literal) throws SQLException {
       throw new UnsupportedOperationException();
     }
@@ -934,6 +1028,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PreferQueryMode getPreferQueryMode() {
       throw new UnsupportedOperationException();
     }
@@ -941,6 +1036,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public AutoSave getAutosave() {
       throw new UnsupportedOperationException();
     }
@@ -948,6 +1044,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setAutosave(AutoSave autoSave) {
       throw new UnsupportedOperationException();
     }
@@ -955,6 +1052,7 @@ public abstract class AbstractArraysTest<A> {
     /**
      * {@inheritDoc}
      */
+    @Override
     public PGReplicationConnection getReplicationAPI() {
       throw new UnsupportedOperationException();
     }
