@@ -5,14 +5,16 @@
 
 package org.postgresql.test.core;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.fail;
+
 import org.postgresql.PGProperty;
 import org.postgresql.core.ServerVersion;
 import org.postgresql.test.TestUtil;
 import org.postgresql.util.PSQLState;
 
-import org.junit.Assert;
-import org.junit.Assume;
-import org.junit.Test;
+import org.junit.jupiter.api.Assumptions;
+import org.junit.jupiter.api.Test;
 
 import java.sql.Connection;
 import java.sql.PreparedStatement;
@@ -20,7 +22,7 @@ import java.sql.SQLException;
 import java.util.Locale;
 import java.util.Properties;
 
-public class LogServerMessagePropertyTest {
+class LogServerMessagePropertyTest {
   private static final String PRIMARY_KEY_NAME = "lms_test_pk";
   private static final String CREATE_TABLE_SQL =
       "CREATE TABLE pg_temp.lms_test ("
@@ -38,7 +40,7 @@ public class LogServerMessagePropertyTest {
    */
   private static String testViolatePrimaryKey(Properties props, boolean batch) throws SQLException {
     Connection conn = TestUtil.openDB(props);
-    Assume.assumeTrue(TestUtil.haveMinimumServerVersion(conn, ServerVersion.v9_1));
+    Assumptions.assumeTrue(TestUtil.haveMinimumServerVersion(conn, ServerVersion.v9_1));
     try {
       TestUtil.execute(conn, CREATE_TABLE_SQL);
       if (batch) {
@@ -53,13 +55,13 @@ public class LogServerMessagePropertyTest {
         TestUtil.execute(conn, INSERT_SQL);
       }
     } catch (SQLException e) {
-      Assert.assertEquals("SQL state must be for a unique violation", PSQLState.UNIQUE_VIOLATION.getState(), e.getSQLState());
+      assertEquals(PSQLState.UNIQUE_VIOLATION.getState(), e.getSQLState(), "SQL state must be for a unique violation");
       return e.getMessage();
     } finally {
       conn.close();
     }
     // Should never get here:
-    Assert.fail("A duplicate key exception should have occurred");
+    fail("A duplicate key exception should have occurred");
     return null;
   }
 
@@ -69,18 +71,18 @@ public class LogServerMessagePropertyTest {
 
   private static void assertMessageContains(String message, String text) {
     if (!message.toLowerCase(Locale.ROOT).contains(text.toLowerCase(Locale.ROOT))) {
-      Assert.fail(String.format("Message must contain text '%s': %s", text, message));
+      fail(String.format("Message must contain text '%s': %s", text, message));
     }
   }
 
   private static void assertMessageDoesNotContain(String message, String text) {
     if (message.toLowerCase(Locale.ROOT).contains(text.toLowerCase(Locale.ROOT))) {
-      Assert.fail(String.format("Message must not contain text '%s': %s", text, message));
+      fail(String.format("Message must not contain text '%s': %s", text, message));
     }
   }
 
   @Test
-  public void testWithDefaults() throws SQLException {
+  void withDefaults() throws SQLException {
     Properties props = new Properties();
     String message = testViolatePrimaryKey(props);
     assertMessageContains(message, PRIMARY_KEY_NAME);
@@ -92,7 +94,7 @@ public class LogServerMessagePropertyTest {
    * NOTE: This should be the same as the default case as "true" is the default.
    */
   @Test
-  public void testWithExplicitlyEnabled() throws SQLException {
+  void withExplicitlyEnabled() throws SQLException {
     Properties props = new Properties();
     props.setProperty(PGProperty.LOG_SERVER_ERROR_DETAIL.getName(), "true");
     String message = testViolatePrimaryKey(props);
@@ -102,7 +104,7 @@ public class LogServerMessagePropertyTest {
   }
 
   @Test
-  public void testWithLogServerErrorDetailDisabled() throws SQLException {
+  void withLogServerErrorDetailDisabled() throws SQLException {
     Properties props = new Properties();
     props.setProperty(PGProperty.LOG_SERVER_ERROR_DETAIL.getName(), "false");
     String message = testViolatePrimaryKey(props);
@@ -112,7 +114,7 @@ public class LogServerMessagePropertyTest {
   }
 
   @Test
-  public void testBatchWithDefaults() throws SQLException {
+  void batchWithDefaults() throws SQLException {
     Properties props = new Properties();
     String message = testViolatePrimaryKey(props, true);
     assertMessageContains(message, PRIMARY_KEY_NAME);
@@ -124,7 +126,7 @@ public class LogServerMessagePropertyTest {
    * NOTE: This should be the same as the default case as "true" is the default.
    */
   @Test
-  public void testBatchExplicitlyEnabled() throws SQLException {
+  void batchExplicitlyEnabled() throws SQLException {
     Properties props = new Properties();
     props.setProperty(PGProperty.LOG_SERVER_ERROR_DETAIL.getName(), "true");
     String message = testViolatePrimaryKey(props, true);
@@ -134,7 +136,7 @@ public class LogServerMessagePropertyTest {
   }
 
   @Test
-  public void testBatchWithLogServerErrorDetailDisabled() throws SQLException {
+  void batchWithLogServerErrorDetailDisabled() throws SQLException {
     Properties props = new Properties();
     props.setProperty(PGProperty.LOG_SERVER_ERROR_DETAIL.getName(), "false");
     String message = testViolatePrimaryKey(props, true);
