@@ -5,16 +5,16 @@
 
 package org.postgresql.test.jdbc2;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import org.postgresql.core.Encoding;
 import org.postgresql.test.TestUtil;
 
-import org.junit.After;
-import org.junit.Assume;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assumptions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 import java.sql.Connection;
 import java.sql.PreparedStatement;
@@ -23,20 +23,20 @@ import java.sql.Statement;
 import java.util.Arrays;
 
 /*
- * Test case for various encoding problems.
- *
- * Ensure that we can do a round-trip of all server-supported unicode values without trashing them,
- * and that bad encodings are detected.
- */
-public class DatabaseEncodingTest {
+* Test case for various encoding problems.
+*
+* Ensure that we can do a round-trip of all server-supported unicode values without trashing them,
+* and that bad encodings are detected.
+*/
+class DatabaseEncodingTest {
   private static final int STEP = 100;
 
   private Connection con;
 
   // Set up the fixture for this testcase: a connection to a database with
   // a table for this test.
-  @Before
-  public void setUp() throws Exception {
+  @BeforeEach
+  void setUp() throws Exception {
     con = TestUtil.openDB();
     TestUtil.createTempTable(con, "testdbencoding",
         "unicode_ordinal integer primary key not null, unicode_string varchar(" + STEP + ")");
@@ -46,15 +46,15 @@ public class DatabaseEncodingTest {
   }
 
   // Tear down the fixture for this test case.
-  @After
-  public void tearDown() throws Exception {
+  @AfterEach
+  void tearDown() throws Exception {
     con.setAutoCommit(true);
     TestUtil.closeDB(con);
   }
 
   private static String dumpString(String s) {
     StringBuffer sb = new StringBuffer(s.length() * 6);
-    for (int i = 0; i < s.length(); ++i) {
+    for (int i = 0; i < s.length(); i++) {
       sb.append("\\u");
       char c = s.charAt(i);
       sb.append(Integer.toHexString((c >> 12) & 15));
@@ -66,9 +66,9 @@ public class DatabaseEncodingTest {
   }
 
   @Test
-  public void testEncoding() throws Exception {
+  void encoding() throws Exception {
     String databaseEncoding = TestUtil.queryForString(con, "SELECT getdatabaseencoding()");
-    Assume.assumeTrue("Database encoding must be UTF8", databaseEncoding.equals("UTF8"));
+    Assumptions.assumeTrue("UTF8".equals(databaseEncoding), "Database encoding must be UTF8");
 
     boolean testHighUnicode = true;
 
@@ -79,7 +79,7 @@ public class DatabaseEncodingTest {
     for (int i = 1; i < 0xd800; i += STEP) {
       int count = (i + STEP) > 0xd800 ? 0xd800 - i : STEP;
       char[] testChars = new char[count];
-      for (int j = 0; j < count; ++j) {
+      for (int j = 0; j < count; j++) {
         testChars[j] = (char) (i + j);
       }
 
@@ -93,7 +93,7 @@ public class DatabaseEncodingTest {
     for (int i = 0xe000; i < 0x10000; i += STEP) {
       int count = (i + STEP) > 0x10000 ? 0x10000 - i : STEP;
       char[] testChars = new char[count];
-      for (int j = 0; j < count; ++j) {
+      for (int j = 0; j < count; j++) {
         testChars[j] = (char) (i + j);
       }
 
@@ -108,7 +108,7 @@ public class DatabaseEncodingTest {
       for (int i = 0x10000; i < 0x110000; i += STEP) {
         int count = (i + STEP) > 0x110000 ? 0x110000 - i : STEP;
         char[] testChars = new char[count * 2];
-        for (int j = 0; j < count; ++j) {
+        for (int j = 0; j < count; j++) {
           testChars[j * 2] = (char) (0xd800 + ((i + j - 0x10000) >> 10));
           testChars[j * 2 + 1] = (char) (0xdc00 + ((i + j - 0x10000) & 0x3ff));
         }
@@ -137,14 +137,15 @@ public class DatabaseEncodingTest {
 
       int count = (i + STEP) > 0xd800 ? 0xd800 - i : STEP;
       char[] testChars = new char[count];
-      for (int j = 0; j < count; ++j) {
+      for (int j = 0; j < count; j++) {
         testChars[j] = (char) (i + j);
       }
 
       String testString = new String(testChars);
 
-      assertEquals("Test string: " + dumpString(testString), dumpString(testString),
-          dumpString(rs.getString(2)));
+      assertEquals(dumpString(testString),
+          dumpString(rs.getString(2)),
+          "Test string: " + dumpString(testString));
     }
 
     for (int i = 0xe000; i < 0x10000; i += STEP) {
@@ -153,14 +154,15 @@ public class DatabaseEncodingTest {
 
       int count = (i + STEP) > 0x10000 ? 0x10000 - i : STEP;
       char[] testChars = new char[count];
-      for (int j = 0; j < count; ++j) {
+      for (int j = 0; j < count; j++) {
         testChars[j] = (char) (i + j);
       }
 
       String testString = new String(testChars);
 
-      assertEquals("Test string: " + dumpString(testString), dumpString(testString),
-          dumpString(rs.getString(2)));
+      assertEquals(dumpString(testString),
+          dumpString(rs.getString(2)),
+          "Test string: " + dumpString(testString));
     }
 
     if (testHighUnicode) {
@@ -170,26 +172,27 @@ public class DatabaseEncodingTest {
 
         int count = (i + STEP) > 0x110000 ? 0x110000 - i : STEP;
         char[] testChars = new char[count * 2];
-        for (int j = 0; j < count; ++j) {
+        for (int j = 0; j < count; j++) {
           testChars[j * 2] = (char) (0xd800 + ((i + j - 0x10000) >> 10));
           testChars[j * 2 + 1] = (char) (0xdc00 + ((i + j - 0x10000) & 0x3ff));
         }
 
         String testString = new String(testChars);
 
-        assertEquals("Test string: " + dumpString(testString), dumpString(testString),
-            dumpString(rs.getString(2)));
+        assertEquals(dumpString(testString),
+            dumpString(rs.getString(2)),
+            "Test string: " + dumpString(testString));
       }
     }
   }
 
   @Test
-  public void testUTF8Decode() throws Exception {
+  void uTF8Decode() throws Exception {
     // Tests for our custom UTF-8 decoder.
 
     Encoding utf8Encoding = Encoding.getJVMEncoding("UTF-8");
 
-    for (int ch = 0; ch < 0x110000; ++ch) {
+    for (int ch = 0; ch < 0x110000; ch++) {
       if (ch >= 0xd800 && ch < 0xe000) {
         continue; // Surrogate range.
       }
@@ -206,10 +209,12 @@ public class DatabaseEncodingTest {
       String jvmDecoding = new String(jvmEncoding, 0, jvmEncoding.length, "UTF-8");
       String ourDecoding = utf8Encoding.decode(jvmEncoding, 0, jvmEncoding.length);
 
-      assertEquals("Test string: " + dumpString(testString), dumpString(testString),
-          dumpString(jvmDecoding));
-      assertEquals("Test string: " + dumpString(testString), dumpString(testString),
-          dumpString(ourDecoding));
+      assertEquals(dumpString(testString),
+          dumpString(jvmDecoding),
+          "Test string: " + dumpString(testString));
+      assertEquals(dumpString(testString),
+          dumpString(ourDecoding),
+          "Test string: " + dumpString(testString));
     }
   }
 
@@ -217,7 +222,7 @@ public class DatabaseEncodingTest {
    * Tests that invalid utf-8 values are replaced with the unicode replacement chart.
    */
   @Test
-  public void testTruncatedUTF8Decode() throws Exception {
+  void truncatedUTF8Decode() throws Exception {
     Encoding utf8Encoding = Encoding.getJVMEncoding("UTF-8");
 
     byte[][] shortSequences = new byte[][]{{(byte) 0xc0}, // Second byte must be present
@@ -231,22 +236,22 @@ public class DatabaseEncodingTest {
     };
 
     byte[] paddedSequence = new byte[32];
-    for (int i = 0; i < shortSequences.length; ++i) {
+    for (int i = 0; i < shortSequences.length; i++) {
       byte[] sequence = shortSequences[i];
       String expected = "\uFFFD";
-      for (int j = 1; j < sequence.length; ++j) {
+      for (int j = 1; j < sequence.length; j++) {
         expected += "\uFFFD";
       }
 
       String str = utf8Encoding.decode(sequence, 0, sequence.length);
-      assertEquals("itr:" + i, expected, str);
+      assertEquals(expected, str, "itr:" + i);
 
       // Try it with padding and a truncated length.
       Arrays.fill(paddedSequence, (byte) 0);
       System.arraycopy(sequence, 0, paddedSequence, 0, sequence.length);
 
       str = utf8Encoding.decode(paddedSequence, 0, sequence.length);
-      assertEquals("itr:" + i, expected, str);
+      assertEquals(expected, str, "itr:" + i);
     }
   }
 }
