@@ -8,15 +8,20 @@ package org.postgresql.test.jdbc2;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
+import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 
 import org.postgresql.PGStatement;
 import org.postgresql.core.ServerVersion;
+import org.postgresql.jdbc.PgConnection;
 import org.postgresql.jdbc.PgStatement;
 import org.postgresql.jdbc.PreferQueryMode;
 import org.postgresql.test.TestUtil;
 import org.postgresql.test.util.BrokenInputStream;
+import org.postgresql.util.GT;
+import org.postgresql.util.PSQLState;
 
 import org.junit.Assert;
 import org.junit.Assume;
@@ -26,6 +31,7 @@ import org.junit.runners.Parameterized;
 
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
+import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
@@ -109,7 +115,6 @@ public class PreparedStatementTest extends BaseTest4 {
 
   @Test
   public void testSetBinaryStream() throws SQLException {
-    assumeByteaSupported();
     ByteArrayInputStream bais;
     byte[] buf = new byte[10];
     for (int i = 0; i < buf.length; i++) {
@@ -172,7 +177,7 @@ public class PreparedStatementTest extends BaseTest4 {
   }
 
   @Test
-  public void testBinaryStreamErrorsRestartable() throws SQLException {
+  public void testBinaryStreamErrorsRestartable() throws SQLException, IOException {
     byte[] buf = new byte[10];
     for (int i = 0; i < buf.length; i++) {
       buf[i] = (byte) i;
@@ -195,25 +200,24 @@ public class PreparedStatementTest extends BaseTest4 {
     runBrokenStream(is, Integer.MAX_VALUE);
   }
 
-  private void runBrokenStream(InputStream is, int length) throws SQLException {
-    PreparedStatement pstmt = null;
-    try {
-      pstmt = con.prepareStatement("INSERT INTO streamtable (bin,str) VALUES (?,?)");
-      pstmt.setBinaryStream(1, is, length);
-      pstmt.setString(2, "Other");
-      pstmt.executeUpdate();
-      fail("This isn't supposed to work.");
-    } catch (SQLException sqle) {
-      // don't need to rollback because we're in autocommit mode
-      pstmt.close();
-
-      // verify the connection is still valid and the row didn't go in.
-      Statement stmt = con.createStatement();
-      ResultSet rs = stmt.executeQuery("SELECT COUNT(*) FROM streamtable");
+  private void runBrokenStream(InputStream is, int length) throws SQLException, IOException {
+    assertThrows(
+        "the provided stream length is " + length
+            + ", and the number of available bytes on stream length is " + is.available()
+            + ", so the bind should fail",
+        SQLException.class, () -> {
+          try (PreparedStatement pstmt =
+                   con.prepareStatement("INSERT INTO streamtable (bin,str) VALUES (?,?)");) {
+            pstmt.setBinaryStream(1, is, length);
+            pstmt.setString(2, "Other");
+            pstmt.executeUpdate();
+          }
+        });
+    // verify the connection is still valid and the row didn't go in.
+    try (Statement stmt = con.createStatement();
+         ResultSet rs = stmt.executeQuery("SELECT COUNT(*) FROM streamtable");) {
       assertTrue(rs.next());
       assertEquals(0, rs.getInt(1));
-      rs.close();
-      stmt.close();
     }
   }
 
@@ -235,6 +239,173 @@ public class PreparedStatementTest extends BaseTest4 {
     pstmt.close();
   }
 
+  @Test
+  public void ByteArrayInputStream_setBinaryStream_toString() throws SQLException {
+    try (PreparedStatement pstmt =
+             con.prepareStatement("INSERT INTO streamtable VALUES (?,?)")) {
+
+      byte[] buf = new byte[]{0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
+      ByteArrayInputStream byteStream = new ByteArrayInputStream(buf);
+
+      pstmt.setBinaryStream(1, byteStream, buf.length);
+      assertEquals(
+          "InputStream parameter should come as ?, and the second parameter is unset, so it should be ? as well",
+          "INSERT INTO streamtable VALUES (?,?)",
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed with half-set parameters"));
+
+      pstmt.setString(2, "test");
+
+      String expected = "INSERT INTO streamtable VALUES (?,('test'))";
+      assertEquals(
+          "InputStream parameter should come as ? when calling PreparedStatement#toString as we can't process input stream twice yet",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed after setting parameters"));
+
+      assertEquals(
+          "InputStream parameter should come as ? when calling PreparedStatement#toString as we can't process input stream twice yet",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "Second PreparedStatement#toString call should succeed as well"));
+
+      pstmt.execute();
+
+      assertEquals(
+          "PreparedStatement#toString after .execute()",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed even after execute()"));
+    }
+  }
+
+  @Test
+  public void ByteArrayInputStream_setBinaryStream_addBatch_toString() throws SQLException {
+    try (PreparedStatement pstmt =
+             con.prepareStatement("INSERT INTO streamtable VALUES (?,?)")) {
+
+      pstmt.setBinaryStream(1, new ByteArrayInputStream(new byte[]{0, 1}), 2);
+      pstmt.setString(2, "line1");
+      pstmt.addBatch();
+      pstmt.setBinaryStream(1, new ByteArrayInputStream(new byte[]{0, 1, 2}), 3);
+      pstmt.setString(2, "line2");
+      pstmt.addBatch();
+
+      String expected = "INSERT INTO streamtable VALUES (?,('line1'));\n"
+          + "INSERT INTO streamtable VALUES (?,('line2'))";
+      assertEquals(
+          "InputStream parameter should come as ? when calling PreparedStatement#toString as we can't process input stream twice yet",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed after addBatch"));
+
+      assertEquals(
+          "InputStream parameter should come as ? when calling PreparedStatement#toString as we can't process input stream twice yet",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "Second PreparedStatement#toString call should succeed as well"));
+
+      pstmt.executeBatch();
+
+      assertEquals(
+          "PreparedStatement#toString after executeBatch() seem to equal to the latest parameter row",
+          "INSERT INTO streamtable VALUES (?,('line2'))",
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed even after executeBatch()"));
+    }
+  }
+
+  @Test
+  public void ByteArray_setBytes_toString() throws SQLException {
+    try (PreparedStatement pstmt =
+             con.prepareStatement("INSERT INTO streamtable VALUES (?,?)")) {
+
+      byte[] buf = new byte[]{0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
+
+      pstmt.setBytes(1, buf);
+
+      assertEquals(
+          "byte[] parameter could be rendered, and the second parameter is unset, so it should be ? as well",
+          "INSERT INTO streamtable VALUES ('\\x00010203040506070809'::bytea,?)",
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed with half-set parameters"));
+
+      pstmt.setString(2, "test");
+
+      String expected = "INSERT INTO streamtable VALUES ('\\x00010203040506070809'::bytea,('test'))";
+      assertEquals(
+          "byte[] should be rendered when calling PreparedStatement#toString",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed after setting parameters"));
+
+      assertEquals(
+          "byte[] should be rendered when calling PreparedStatement#toString",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "Second PreparedStatement#toString call should succeed as well"));
+
+      pstmt.execute();
+
+      assertEquals(
+          "PreparedStatement#toString after .execute()",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed even after execute()"));
+    }
+  }
+
+  @Test
+  public void ByteArray_setBytes_addBatch_toString() throws SQLException {
+    try (PreparedStatement pstmt =
+             con.prepareStatement("INSERT INTO streamtable VALUES (?,?)")) {
+
+      pstmt.setBytes(1, new byte[]{0, 1});
+      pstmt.setString(2, "line1");
+      pstmt.addBatch();
+
+      pstmt.setBytes(1, new byte[]{0, 1, 2});
+      pstmt.setString(2, "line2");
+      pstmt.addBatch();
+
+      String expected = "INSERT INTO streamtable VALUES ('\\x0001'::bytea,('line1'));\n"
+          + "INSERT INTO streamtable VALUES ('\\x000102'::bytea,('line2'))";
+      assertEquals(
+          "byte[] should be rendered when calling PreparedStatement#toString",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed after addBatch"));
+
+      assertEquals(
+          "byte[] should be rendered when calling PreparedStatement#toString",
+          expected,
+          assertDoesNotThrow(
+              pstmt::toString,
+              "Second PreparedStatement#toString call should succeed as well"));
+
+      pstmt.executeBatch();
+
+      assertEquals(
+          "PreparedStatement#toString after executeBatch() seem to equal to the latest parameter row",
+          "INSERT INTO streamtable VALUES ('\\x000102'::bytea,('line2'))",
+          assertDoesNotThrow(
+              pstmt::toString,
+              "PreparedStatement#toString call should succeed even after executeBatch()"));
+    }
+  }
+
   @Test
   public void testTrailingSpaces() throws SQLException {
     PreparedStatement pstmt =
@@ -634,7 +805,7 @@ public class PreparedStatementTest extends BaseTest4 {
     checkNaNLiterals(stmt, stmt.executeQuery());
   }
 
-  private void checkNaNLiterals(Statement stmt, ResultSet rs) throws SQLException {
+  private static void checkNaNLiterals(Statement stmt, ResultSet rs) throws SQLException {
     rs.next();
     assertTrue("Double.isNaN((Double) rs.getObject", Double.isNaN((Double) rs.getObject(3)));
     assertTrue("Double.isNaN(rs.getDouble", Double.isNaN(rs.getDouble(3)));
@@ -642,36 +813,117 @@ public class PreparedStatementTest extends BaseTest4 {
     assertTrue("Float.isNaN(rs.getFloat", Float.isNaN(rs.getFloat(2)));
     assertTrue("Double.isNaN((Double) rs.getObject", Double.isNaN((Double) rs.getObject(1)));
     assertTrue("Double.isNaN(rs.getDouble", Double.isNaN(rs.getDouble(1)));
+    try {
+      rs.getBigDecimal(1);
+      fail("NaN::numeric rs.getBigDecimal");
+    } catch (SQLException e) {
+      assertEquals(PSQLState.NUMERIC_VALUE_OUT_OF_RANGE.getState(), e.getSQLState());
+      assertEquals(GT.tr("Bad value for type {0} : {1}", "BigDecimal", "NaN"), e.getMessage());
+    }
+
     rs.close();
     stmt.close();
   }
 
   @Test
-  public void testNaNSetDoubleFloat() throws SQLException {
-    PreparedStatement ps = con.prepareStatement("select ?, ?");
+  public void testInfinityLiteralsSimpleStatement() throws SQLException {
+    assumeMinimumServerVersion("v14 introduced 'Infinity'::numeric", ServerVersion.v14);
+
+    String query = "SELECT 'Infinity'::numeric, 'Infinity'::real, 'Infinity'::double precision, "
+        + "'-Infinity'::numeric, '-Infinity'::real, '-Infinity'::double precision";
+    try (Statement stmt = con.createStatement();
+         ResultSet rs = stmt.executeQuery(query)) {
+      checkInfinityLiterals(rs);
+    }
+  }
+
+  @Test
+  public void testInfinityLiteralsPreparedStatement() throws SQLException {
+    assumeMinimumServerVersion("v14 introduced 'Infinity'::numeric", ServerVersion.v14);
+
+    String query = "SELECT 'Infinity'::numeric, 'Infinity'::real, 'Infinity'::double precision, "
+        + "'-Infinity'::numeric, '-Infinity'::real, '-Infinity'::double precision";
+    try (PreparedStatement stmt = con.prepareStatement(query);
+         ResultSet rs = stmt.executeQuery()) {
+      checkInfinityLiterals(rs);
+    }
+  }
+
+  private static void checkInfinityLiterals(ResultSet rs) throws SQLException {
+    rs.next();
+    assertEquals("inf numeric rs.getObject", Double.POSITIVE_INFINITY, rs.getObject(1));
+    assertEquals("inf numeric rs.getDouble", Double.POSITIVE_INFINITY, rs.getDouble(1), 0.0);
+    assertEquals("inf real rs.getObject", Float.POSITIVE_INFINITY, rs.getObject(2));
+    assertEquals("inf real rs.getFloat", Float.POSITIVE_INFINITY, rs.getFloat(2), 0.0);
+    assertEquals("inf double precision rs.getObject", Double.POSITIVE_INFINITY, rs.getObject(3));
+    assertEquals("inf double precision rs.getDouble", Double.POSITIVE_INFINITY, rs.getDouble(3), 0.0);
+
+    assertEquals("-inf numeric rs.getObject", Double.NEGATIVE_INFINITY, rs.getObject(4));
+    assertEquals("-inf numeric rs.getDouble", Double.NEGATIVE_INFINITY, rs.getDouble(4), 0.0);
+    assertEquals("-inf real rs.getObject", Float.NEGATIVE_INFINITY, rs.getObject(5));
+    assertEquals("-inf real rs.getFloat", Float.NEGATIVE_INFINITY, rs.getFloat(5), 0.0);
+    assertEquals("-inf double precision rs.getObject", Double.NEGATIVE_INFINITY, rs.getObject(6));
+    assertEquals("-inf double precision rs.getDouble", Double.NEGATIVE_INFINITY, rs.getDouble(6), 0.0);
+
+    try {
+      rs.getBigDecimal(1);
+      fail("inf numeric rs.getBigDecimal");
+    } catch (SQLException e) {
+      assertEquals(PSQLState.NUMERIC_VALUE_OUT_OF_RANGE.getState(), e.getSQLState());
+      assertEquals(GT.tr("Bad value for type {0} : {1}", "BigDecimal", "Infinity"), e.getMessage());
+    }
+
+    try {
+      rs.getBigDecimal(4);
+      fail("-inf numeric rs.getBigDecimal");
+    } catch (SQLException e) {
+      assertEquals(PSQLState.NUMERIC_VALUE_OUT_OF_RANGE.getState(), e.getSQLState());
+      assertEquals(GT.tr("Bad value for type {0} : {1}", "BigDecimal", "-Infinity"), e.getMessage());
+    }
+  }
+
+  @Test
+  public void testSpecialSetDoubleFloat() throws SQLException {
+    PreparedStatement ps = con.prepareStatement("select ?, ?, ?, ?, ?, ?");
     ps.setFloat(1, Float.NaN);
     ps.setDouble(2, Double.NaN);
+    ps.setFloat(3, Float.POSITIVE_INFINITY);
+    ps.setDouble(4, Double.POSITIVE_INFINITY);
+    ps.setFloat(5, Float.NEGATIVE_INFINITY);
+    ps.setDouble(6, Double.NEGATIVE_INFINITY);
 
     checkNaNParams(ps);
   }
 
   @Test
-  public void testNaNSetObject() throws SQLException {
-    PreparedStatement ps = con.prepareStatement("select ?, ?");
+  public void testSpecialSetObject() throws SQLException {
+    PreparedStatement ps = con.prepareStatement("select ?, ?, ?, ?, ?, ?");
     ps.setObject(1, Float.NaN);
     ps.setObject(2, Double.NaN);
+    ps.setObject(3, Float.POSITIVE_INFINITY);
+    ps.setObject(4, Double.POSITIVE_INFINITY);
+    ps.setObject(5, Float.NEGATIVE_INFINITY);
+    ps.setObject(6, Double.NEGATIVE_INFINITY);
 
     checkNaNParams(ps);
   }
 
-  private void checkNaNParams(PreparedStatement ps) throws SQLException {
+  private static void checkNaNParams(PreparedStatement ps) throws SQLException {
     ResultSet rs = ps.executeQuery();
     rs.next();
 
     assertTrue("Float.isNaN((Float) rs.getObject", Float.isNaN((Float) rs.getObject(1)));
     assertTrue("Float.isNaN(rs.getFloat", Float.isNaN(rs.getFloat(1)));
+    assertTrue("Double.isNaN((Double) rs.getObject", Double.isNaN((Double) rs.getObject(2)));
     assertTrue("Double.isNaN(rs.getDouble", Double.isNaN(rs.getDouble(2)));
-    assertTrue("Double.isNaN(rs.getDouble", Double.isNaN(rs.getDouble(2)));
+    assertEquals("Float.POSITIVE_INFINITY rs.getObject", Float.POSITIVE_INFINITY, rs.getObject(3));
+    assertEquals("Float.POSITIVE_INFINITY rs.getFloat", Float.POSITIVE_INFINITY, rs.getFloat(3), 0);
+    assertEquals("Double.POSITIVE_INFINITY rs.getObject", Double.POSITIVE_INFINITY, rs.getObject(4));
+    assertEquals("Double.POSITIVE_INFINITY rs.getDouble", Double.POSITIVE_INFINITY, rs.getDouble(4), 0);
+    assertEquals("Float.NEGATIVE_INFINITY rs.getObject", Float.NEGATIVE_INFINITY, rs.getObject(5));
+    assertEquals("Float.NEGATIVE_INFINITY rs.getFloat", Float.NEGATIVE_INFINITY, rs.getFloat(5), 0);
+    assertEquals("Double.NEGATIVE_INFINITY rs.getObject", Double.NEGATIVE_INFINITY, rs.getObject(6));
+    assertEquals("Double.NEGATIVE_INFINITY rs.getDouble", Double.NEGATIVE_INFINITY, rs.getDouble(6), 0);
 
     TestUtil.closeQuietly(rs);
     TestUtil.closeQuietly(ps);
@@ -1394,6 +1646,8 @@ public class PreparedStatementTest extends BaseTest4 {
       pstmt.executeBatch();
     }
     pstmt.close();
+    Assume.assumeFalse("Test assertions below support only non-rewritten insert statements",
+        con.unwrap(PgConnection.class).getQueryExecutor().isReWriteBatchedInsertsEnabled());
     assertTrue("prepareThreshold=5, so the statement should be server-prepared",
         ((PGStatement) pstmt).isUseServerPrepare());
     assertEquals("prepareThreshold=5, so the statement should be server-prepared", 1,
