@@ -8,14 +8,17 @@ package org.postgresql.test.jdbc2;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.junit.Assume.assumeTrue;
 
+import org.postgresql.PGConnection;
 import org.postgresql.core.ServerVersion;
 import org.postgresql.jdbc.PreferQueryMode;
 import org.postgresql.test.TestUtil;
 import org.postgresql.util.PGobject;
+import org.postgresql.util.PSQLException;
 
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -936,13 +939,62 @@ public class ResultSetTest extends BaseTest4 {
 
     assertEquals(ResultSet.CONCUR_UPDATABLE, rs.getConcurrency());
     assertEquals(ResultSet.TYPE_SCROLL_SENSITIVE, rs.getType());
-    assertEquals(100, rs.getFetchSize());
+    if (!con.unwrap(PGConnection.class).getAdaptiveFetch()) {
+      assertEquals("ResultSet.fetchSize should not change after query execution",
+          100,
+          rs.getFetchSize());
+    }
     assertEquals(ResultSet.FETCH_UNKNOWN, rs.getFetchDirection());
 
     rs.close();
     stmt.close();
   }
 
+  @Test
+  public void testCreateStatementWithInvalidResultSetParams() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.createStatement(-1, -1,-1));
+  }
+
+  @Test
+  public void testCreateStatementWithInvalidResultSetConcurrency() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.createStatement( ResultSet.TYPE_SCROLL_INSENSITIVE, -1) );
+  }
+
+  @Test
+  public void testCreateStatementWithInvalidResultSetHoldability() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.createStatement( ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE, -1) );
+  }
+
+  @Test
+  public void testPrepareStatementWithInvalidResultSetParams() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.prepareStatement("SELECT id FROM testrs", -1, -1,-1));
+  }
+
+  @Test
+  public void testPrepareStatementWithInvalidResultSetConcurrency() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.prepareStatement("SELECT id FROM testrs", ResultSet.TYPE_SCROLL_INSENSITIVE, -1) );
+  }
+
+  @Test
+  public void testPrepareStatementWithInvalidResultSetHoldability() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.prepareStatement("SELECT id FROM testrs", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE, -1) );
+  }
+
+  @Test
+  public void testPrepareCallWithInvalidResultSetParams() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.prepareCall("SELECT id FROM testrs", -1, -1,-1));
+  }
+
+  @Test
+  public void testPrepareCallWithInvalidResultSetConcurrency() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.prepareCall("SELECT id FROM testrs", ResultSet.TYPE_SCROLL_INSENSITIVE, -1) );
+  }
+
+  @Test
+  public void testPrepareCallWithInvalidResultSetHoldability() throws SQLException {
+    assertThrows(PSQLException.class, () -> con.prepareCall("SELECT id FROM testrs", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE, -1) );
+  }
+
   @Test
   public void testZeroRowResultPositioning() throws SQLException {
     Statement stmt =
@@ -1339,7 +1391,7 @@ public class ResultSetTest extends BaseTest4 {
   }
 
   @SuppressWarnings("unchecked")
-  private Map<String, Integer> getResultSetColumnNameIndexMap(ResultSet stmt) {
+  private static Map<String, Integer> getResultSetColumnNameIndexMap(ResultSet stmt) {
     try {
       Field columnNameIndexMapField = stmt.getClass().getDeclaredField("columnNameIndexMap");
       columnNameIndexMapField.setAccessible(true);
