@@ -85,15 +85,6 @@ class StatementTest {
     con.close();
   }
 
-  private void assumeLongTest() {
-    // Run the test:
-    //   Travis: in PG_VERSION=HEAD
-    //   Other: always
-    if ("true".equals(System.getenv("TRAVIS"))) {
-      Assumptions.assumeTrue("HEAD".equals(System.getenv("PG_VERSION")));
-    }
-  }
-
   @Test
   void close() throws SQLException {
     Statement stmt = con.createStatement();
@@ -536,8 +527,8 @@ class StatementTest {
   }
 
   /**
-   * <p>Demonstrates a safe approach to concurrently reading the latest
-   * warnings while periodically clearing them.</p>
+   * Demonstrates a safe approach to concurrently reading the latest
+   * warnings while periodically clearing them.
    *
    * <p>One drawback of this approach is that it requires the reader to make it to the end of the
    * warning chain before clearing it, so long as your warning processing step is not very slow,
@@ -739,7 +730,6 @@ class StatementTest {
    */
   @Test
   void shortQueryTimeout() throws SQLException {
-    assumeLongTest();
 
     long deadLine = System.nanoTime() + TimeUnit.SECONDS.toNanos(10);
     Statement stmt = con.createStatement();
@@ -872,7 +862,8 @@ class StatementTest {
 
     try (StrangeProxyServer proxyServer = new StrangeProxyServer(TestUtil.getServer(), TestUtil.getPort())) {
       Properties props = new Properties();
-      props.setProperty(TestUtil.SERVER_HOST_PORT_PROP, String.format("%s:%s", "localhost", proxyServer.getServerPort()));
+      TestUtil.setTestUrlProperty(props, PGProperty.PG_HOST, "localhost");
+      TestUtil.setTestUrlProperty(props, PGProperty.PG_PORT, String.valueOf(proxyServer.getServerPort()));
       PGProperty.CANCEL_SIGNAL_TIMEOUT.set(props, 1);
 
       try (Connection conn = TestUtil.openDB(props); Statement stmt = conn.createStatement()) {
@@ -891,9 +882,12 @@ class StatementTest {
     executor.shutdownNow();
   }
 
-  @Test
-  @Timeout(10)
-  void closeInProgressStatement() throws Exception {
+  /*
+  We are going to use this test to test version 3.2 since the only change in 3.2 is the width of the
+  cancel key. We need a test that does a cancel. We call this below once without changing the
+  protocol version and once with protocol version 3.2
+   */
+  private void closePrivateInProgressStatement() throws Exception {
     ExecutorService executor = Executors.newSingleThreadExecutor();
     final Connection outerLockCon = TestUtil.openDB();
     outerLockCon.setAutoCommit(false);
@@ -950,6 +944,23 @@ class StatementTest {
     }
   }
 
+  @Test
+  @Timeout(10)
+  void closeInProgressStatement() throws Exception {
+    closePrivateInProgressStatement();
+  }
+
+  @Test
+  @Timeout(10)
+  void closeInProgressStatementProtocol32() throws Exception {
+    Assumptions.assumeTrue(TestUtil.haveMinimumServerVersion(con, ServerVersion.v11));
+    Properties props = new Properties();
+    con.close();
+    PGProperty.PROTOCOL_VERSION.set(props, "3.2");
+    con = TestUtil.openDB(props);
+    closePrivateInProgressStatement();
+  }
+
   @Test
   @Timeout(10)
   void concurrentIsValid() throws Throwable {
