@@ -154,7 +154,7 @@ public class UpdateableResultTest extends BaseTest4 {
     st.close();
   }
 
-  private void checkPositioning(ResultSet rs) throws SQLException {
+  private static void checkPositioning(ResultSet rs) throws SQLException {
     try {
       rs.getInt(1);
       fail("Can't use an incorrectly positioned result set.");
@@ -308,7 +308,6 @@ public class UpdateableResultTest extends BaseTest4 {
 
   @Test
   public void testUpdateStreams() throws SQLException, UnsupportedEncodingException {
-    assumeByteaSupported();
     String string = "Hello";
     byte[] bytes = new byte[]{0, '\\', (byte) 128, (byte) 255};
 
