@@ -7,6 +7,7 @@ package org.postgresql.test.jdbc2.optional;
 
 import org.postgresql.ds.PGSimpleDataSource;
 import org.postgresql.jdbc2.optional.SimpleDataSource;
+import org.postgresql.util.PSQLException;
 
 import org.junit.Test;
 
@@ -21,7 +22,7 @@ public class SimpleDataSourceTest extends BaseDataSourceTest {
    * Creates and configures a new SimpleDataSource.
    */
   @Override
-  protected void initializeDataSource() {
+  protected void initializeDataSource() throws PSQLException {
     if (bds == null) {
       bds = new SimpleDataSource();
       setupDataSource(bds);
