@@ -29,11 +29,10 @@ public class ByteStreamWriterTest extends BaseTest4 {
   @Override
   public void setUp() throws Exception {
     super.setUp();
-    assumeByteaSupported();
     TestUtil.createTempTable(con, "images", "img bytea");
   }
 
-  private ByteBuffer testData(int size) {
+  private static ByteBuffer testData(int size) {
     ByteBuffer data = ByteBuffer.allocate(size);
     Random random = new Random(31459);
     while (data.remaining() > 8) {
